package gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.NoSuchElementException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import contact.Contact;
import contact.ContactAlreadyExistsException;
import contact.Phonebook;
import contact.PhonebookHashMap;

public class PhonebookGUI extends JFrame {

	private static final Font MESSAGE_FONT = new Font(Font.SANS_SERIF, Font.ITALIC, 20);
	private static final Font BUTTON_FONT = new Font(Font.DIALOG, Font.BOLD, 20);
	private static final Font TEXT_FIELD_FONT = new Font(Font.MONOSPACED, Font.BOLD, 20);
	private static final Font LABEL_FONT = new Font(Font.MONOSPACED, Font.BOLD, 30);
	private static final Color MESSAGE_STANDARD_COLOR = Color.BLUE;
	private static final Color MESSAGE_ERROR_COLOR = Color.RED;

	protected JLabel nameLabel = new JLabel("Contact name : ");
	protected JLabel numberLabel = new JLabel("Contact number : ");
	protected JLabel emailLabel = new JLabel("Contact email : ");
	protected JLabel messageLabel = new JLabel("Everything is OK ! ");

	protected JTextField nameField = new JTextField(30);
	protected JTextField numberField = new JTextField(30);
	protected JTextField emailField = new JTextField(30);

	protected JButton addButton = new JButton("Add");
	protected JButton searchNameButton = new JButton("Search by name");
	protected JButton searchNumberButton = new JButton("Search by number");
	protected JButton removeButton = new JButton("Remove");
	protected JButton quitButton = new JButton("Clear");

	private final Phonebook phonebook;

	public PhonebookGUI(String title) {
		super(title);

		// We can choose one of the 3 classes that implement the interface.
		phonebook = new PhonebookHashMap();

		initStyle();

		initLayout();

		initActions();
	}

	protected void initActions() {
		addButton.addActionListener(new AddAction());
		searchNameButton.addActionListener(new SearchByNameAction());
		searchNumberButton.addActionListener(new SearchByNumberAction());
		removeButton.addActionListener(new RemoveAction());
		quitButton.addActionListener(new ClearAction());
	}

	protected void initStyle() {
		nameLabel.setFont(LABEL_FONT);
		numberLabel.setFont(LABEL_FONT);
		emailLabel.setFont(LABEL_FONT);
		nameField.setFont(TEXT_FIELD_FONT);
		numberField.setFont(TEXT_FIELD_FONT);
		emailField.setFont(TEXT_FIELD_FONT);

		addButton.setFont(BUTTON_FONT);
		searchNameButton.setFont(BUTTON_FONT);
		searchNumberButton.setFont(BUTTON_FONT);
		removeButton.setFont(BUTTON_FONT);
		quitButton.setFont(BUTTON_FONT);

		messageLabel.setFont(MESSAGE_FONT);
		messageLabel.setForeground(MESSAGE_STANDARD_COLOR);
	}

	protected void initLayout() {
		GridLayout grid = new GridLayout(4, 3);
		Container contentPane = getContentPane();
		contentPane.setLayout(grid);

		// First line
		contentPane.add(nameLabel);
		contentPane.add(nameField);
		contentPane.add(addButton);

		// Second line
		contentPane.add(numberLabel);
		contentPane.add(numberField);
		contentPane.add(searchNameButton);

		// Third line
		contentPane.add(emailLabel);
		contentPane.add(emailField);
		contentPane.add(searchNumberButton);

		// Fourth line
		contentPane.add(messageLabel);
		contentPane.add(removeButton);
		contentPane.add(quitButton);

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(1000, 200);
		setResizable(false);
		setVisible(true);
	}

	private class AddAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String name = nameField.getText();
			String number = numberField.getText();
			String email = emailField.getText();
			if ((!name.equals("")) && (!number.equals("")) && (!email.equals(""))) {
				Contact contact = new Contact(name, number, email);
				try {
					phonebook.add(contact);
					updateStandardMessage("contact added !");
				} catch (ContactAlreadyExistsException e1) {
					updateErrorMessage(e1.getMessage());
				}
			} else {
				updateErrorMessage("Incomplete information !");
			}
		}

	}

	private class SearchByNumberAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String number = numberField.getText();
			if (!number.equals("")) {
				try {
					Contact contact = phonebook.searchByNumber(number);
					String name = contact.getName();
					String email = contact.getEmail();
					nameField.setText(name);
					emailField.setText(email);
					updateStandardMessage("contact found !");
				} catch (NoSuchElementException e1) {
					updateErrorMessage(e1.getMessage());
				}
			} else {
				updateErrorMessage("Please enter a number !");
			}
		}
	}

	private class SearchByNameAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String name = nameField.getText();
			if (!name.equals("")) {
				try {
					Contact contact = phonebook.searchByName(name);
					String number = contact.getNumber();
					String email = contact.getEmail();
					numberField.setText(number);
					emailField.setText(email);
					updateStandardMessage("contact found !");
				} catch (NoSuchElementException e1) {
					updateErrorMessage(e1.getMessage());
				}
			} else {
				updateErrorMessage("Please enter a name !");
			}
		}

	}

	private class RemoveAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String name = nameField.getText();
			String number = numberField.getText();
			String email = emailField.getText();
			if ((!name.equals("")) && (!number.equals("")) && (!email.equals(""))) {
				// For using equals
				Contact contact = new Contact(name, number, email);
				if (phonebook.containsContact(contact)){
					phonebook.remove(contact);
					updateStandardMessage("contact removed !");
				} else {
					updateErrorMessage("Nothing to be removed !");
				}	
			} else {
				updateErrorMessage("Incomplete information !");
			}
		}

	}

	public class ClearAction implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			nameField.setText("");
			numberField.setText("");
			emailField.setText("");
		}

	}

	private void updateStandardMessage(String message) {
		messageLabel.setForeground(MESSAGE_STANDARD_COLOR);
		messageLabel.setText(message);
	}

	private void updateErrorMessage(String message) {
		messageLabel.setForeground(MESSAGE_ERROR_COLOR);
		messageLabel.setText(message);
	}

	public static void main(String[] args) {
		new PhonebookGUI("Phonebook GUI");
	}

}
